package co.datainsider.flutter

import android.content.Context
import android.os.Bundle
import androidx.annotation.NonNull
import co.datainsider.sdk.Analytics
import co.datainsider.sdk.Configuration
import co.datainsider.sdk.utils.Log
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result

/** DIAnalyticsPlugin */
class DIAnalyticsPlugin: FlutterPlugin, MethodCallHandler {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private lateinit var channel : MethodChannel
  private var applicationContext: Context? = null

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, "co.datainsider.analytics")
    channel.setMethodCallHandler(this)
    applicationContext = flutterPluginBinding.applicationContext
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    var res: Any? = null
    try {
      when (call.method) {
        "configure" -> {
          configure(call.arguments)
          res = false
        }
        "logEvent" -> {
          logEvent(call.arguments)
          res = true
        }
        "addSharedParameters" -> {
          addSharedParameters(call.arguments)
          res = true
        }
        "dispatchEvents" -> {
          dispatchEvents()
          res = true
        }
        "resetAnalyticsData" -> {
          resetAnalyticsData()
          res = true
        }
        "setDebugLogEnabled" -> {
          setDebugLogEnabled(call.arguments)
          res = true
        }
        "setEnabled" -> {
          setEnabled(call.arguments)
          res = true
        }
        "setUser" -> {
          setUser(call.arguments)
          res = true
        }
        "getVersion" -> {
          res = getVersion()
        }
        else -> {
          result.notImplemented()
        }
      }
    } catch (ex: IllegalArgumentException) {
      result.error("error",ex.message, call.arguments)
    }

    if(res != null) {
      result.success(res)
    }
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
    applicationContext = null
  }

  private fun configure(args: Any) {
    if(applicationContext == null) {
      throw IllegalArgumentException("applicationContext null")
    }

    if(args !is Map<*, *>) {
      throw IllegalArgumentException("Invalid argument $args")
    }

    val apiKey: Any? = args["apiKey"]
    if(apiKey !is String) {
      throw IllegalArgumentException("Invalid argument apiKey $apiKey")
    }

    val trackingUrl: Any? = args["trackingUrl"]
    if(trackingUrl !is String) {
      throw IllegalArgumentException("Invalid argument trackingUrl $trackingUrl")
    }

    var config = Configuration(apiKey, trackingUrl)
    val submitInterval: Any? = args["submitInterval"]
    if(submitInterval is Int) {
      config.submitInterval = submitInterval
    }

    val submitTimeoutInterval: Any? = args["submitTimeoutInterval"]
    if(submitTimeoutInterval is Int) {
      config.submitTimeout = submitTimeoutInterval
    }

    val maxSizeOfLogFile: Any? = args["maxSizeOfLogFile"]
    if(maxSizeOfLogFile is Int) {
      config.maxSizeOfLogFile = maxSizeOfLogFile
    }

    val maxNumberOfLogFile: Any? = args["maxNumberOfLogFile"]
    if(maxNumberOfLogFile is Int) {
      config.maxNumberOfLogFile = maxNumberOfLogFile
    }

    val sessionExpireTime: Any? = args["sessionExpireTime"]
    if(sessionExpireTime is Int) {
      config.sessionExpireTime = sessionExpireTime
    }

    val submitLogOnWifiOnly: Any? = args["submitLogOnWifiOnly"]
    if(submitLogOnWifiOnly is Boolean) {
      config.submitLogOnWifiOnly = submitLogOnWifiOnly
    }

    val queueEnabled: Any? = args["queueEnabled"]
    if(queueEnabled is Boolean) {
      config.queueEnabled = queueEnabled
    }

    Analytics.configure(applicationContext!!, config)
  }

  private fun logEvent(args: Any) {
    if(args !is Map<*, *>) {
      throw IllegalArgumentException("Invalid argument $args")
    }
    val name: Any? = args["name"]
    if(name !is String) {
      throw IllegalArgumentException("Invalid argument name $name")
    }

    val params: Any? = args["parameters"]
    if(params !is Map<*, *>?) {
      throw IllegalArgumentException("Invalid argument parameters $name")
    }

    Analytics.logEvent(name, params?.toBundle())
  }

  private  fun addSharedParameters(args: Any) {
    if(args !is Map<*, *>) {
      throw IllegalArgumentException("Invalid argument $args")
    }

    Analytics.addSharedParameters(args.toBundle())
  }

  private fun dispatchEvents() {
    Analytics.dispatchEvents()
  }

  private fun resetAnalyticsData() {
    Analytics.resetAnalyticsData()
  }

  private fun setDebugLogEnabled(args: Any) {
    if (args !is Boolean) {
      throw  IllegalArgumentException("Invalid argument $args")
    }

    Analytics.setDebugLogEnabled(args)
  }

  private fun setEnabled(args: Any) {
    if (args !is Boolean) {
      throw  IllegalArgumentException("Invalid argument $args")
    }

    Analytics.setEnabled(args)
  }

  private fun setUser(args: Any) {
    if(args !is Map<*, *>) {
      throw IllegalArgumentException("Invalid argument $args")
    }
    val userId: Any? = args["userId"]
    if(userId !is String?) {
      throw IllegalArgumentException("Invalid argument userId $userId")
    }

    val info: Any? = args["info"]
    if(info !is Map<*, *>?) {
      throw IllegalArgumentException("Invalid argument info $info")
    }

    Analytics.setUser(userId, info?.toBundle())
  }

  private fun getVersion(): String {
    return Analytics.VERSION
  }
}


fun Map<*, *>.toBundle(): Bundle {
  val bundle = Bundle()
  for(key in this.keys) {
    val value = this[key]
    val keyStr = key.toString()
    if(value == null) {
      bundle.putString(keyStr, null)
      continue
    }

    when (value) {
      is List<*> -> bundle.putString(keyStr, value.toString())
      is Map<*, *> -> bundle.putBundle(keyStr, value.toBundle())
      is String -> bundle.putString(keyStr, value)
      is Int -> bundle.putInt(keyStr, value)
      is Double -> bundle.putDouble(keyStr, value)
      is Long -> bundle.putLong(keyStr, value)
      is Float -> bundle.putFloat(keyStr, value)
      is Boolean -> bundle.putBoolean(keyStr, value)
      is Short -> bundle.putShort(keyStr, value)
      is Byte -> bundle.putByte(keyStr, value)
      is Char -> bundle.putChar(keyStr, value)
      else -> Log.w("Unsupported type ${value.javaClass.name}")
    }
  }

  return bundle
}
