import Flutter
import UIKit
import DIAnalyticsSDK

public class SwiftDIAnalyticsPlugin: NSObject, FlutterPlugin {
  public static func register(with registrar: FlutterPluginRegistrar) {
    let channel = FlutterMethodChannel(name: "co.datainsider.analytics", binaryMessenger: registrar.messenger())
    let instance = SwiftDIAnalyticsPlugin()
    registrar.addMethodCallDelegate(instance, channel: channel)
  }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
    var res: Any?
    do {
      if call.method == "configure" {
        try configure(call.arguments)
      } else if call.method == "logEvent" {
        try logEvent(call.arguments)
      } else if call.method == "setUser" {
        try setUser(call.arguments)
      } else if call.method == "resetAnalyticsData" {
        resetAnalyticsData()
      } else if call.method == "dispatchEvents" {
        dispatchEvents()
      } else if call.method == "addSharedParameters" {
        try addSharedParameters(call.arguments)
      } else if call.method == "setDebugLogEnabled" {
        try setDebugLogEnabled(call.arguments)
      } else if call.method == "setEnabled" {
        try setEnabled(call.arguments)
      } else if call.method == "getVersion" {
        res = getVersion()
      } else {
        throw DIAnalyticsError.unsupportedMethod(name: call.method)
      }
    } catch DIAnalyticsError.unsupportedMethod(let name) {
      res = FlutterError.init(code:"unsupportedMethod", message:"Unsupported Method \(name)", details:nil)
    } catch DIAnalyticsError.invalidArgument(let mesage) {
      res = FlutterError.init(code:"invalidArgument", message:mesage, details:call.arguments)
    } catch let err {
      res = FlutterError.init(code:"unknown error", message:err.localizedDescription, details:call.arguments)
    }
    
    result(res ?? true)
  }
  
  private func configure(_ args: Any?) throws {
    guard let args = args as? Dictionary<String, Any>,
          let apiKey = args["apiKey"] as? String,
          let trackingUrlString = args["trackingUrl"] as? String,
          let trackingUrl = URL(string: trackingUrlString)
    else {
      throw DIAnalyticsError.invalidArgument(message: "Invalid Arguments \(args ?? "null")")
    }
    
    let config = Configuration(apiKey: apiKey, trackingUrl: trackingUrl)
    if let submitInterval = args["submitInterval"] as? Int {
      config.submitInterval = TimeInterval(submitInterval) / 1000
    }

    if let submitTimeoutInterval = args["submitTimeoutInterval"] as? Int {
     config.submitTimeoutInterval = TimeInterval(submitTimeoutInterval) / 1000
   }
    
    if let maxSizeOfLogFile = args["maxSizeOfLogFile"] as? Int {
     config.maxSizeOfLogFile = maxSizeOfLogFile
   }
    if let maxNumberOfLogFile = args["maxNumberOfLogFile"] as? Int {
     config.maxNumberOfLogFile = maxNumberOfLogFile
   }
    if let sessionExpireTime = args["sessionExpireTime"] as? Int {
     config.sessionExpireTime = TimeInterval(sessionExpireTime) / 1000
   }
    if let submitLogOnWifiOnly = args["submitLogOnWifiOnly"] as? Bool {
     config.submitLogOnWifiOnly = submitLogOnWifiOnly
   } 
   if let queueEnabled = args["queueEnabled"] as? Bool {
    config.queueEnabled = queueEnabled
  }
    
    Analytics.configure(config: config)
  }
  
  private func logEvent(_ args: Any?) throws {
    guard let args = args as? Dictionary<String, Any>,
          let name = args["name"] as? String
    else {
      throw DIAnalyticsError.invalidArgument(message: "Invalid Arguments \(args ?? "null")")
    }
    let parameters = args["parameters"] as? Dictionary<String, Any>
    Analytics.logEvent(name, parameters: parameters)
  }
  
  private func setUser(_ args: Any?) throws {
    guard let args = args as? Dictionary<String, Any>
    else {
      throw DIAnalyticsError.invalidArgument(message: "Invalid Arguments \(args ?? "null")")
    }
    
    let userId = args["userId"] as? String
    let info = args["info"] as? Dictionary<String, Any>
    
    Analytics.setUser(userId, info: info)
  }
  
  private func resetAnalyticsData() {
    Analytics.resetAnalyticsData()
  }
  
  private func dispatchEvents() {
    Analytics.dispatchEvents()
  }
  
  private func addSharedParameters(_ args: Any?) throws {
    guard let parameters = args as? Dictionary<String, Any?>
    else {
      throw DIAnalyticsError.invalidArgument(message: "Invalid Arguments \(args ?? "null")")
    }
    
    Analytics.addSharedParameters(parameters)
  }
  
  private func setDebugLogEnabled(_ args: Any?) throws {
    guard let enabled = args as? Bool else {
      throw DIAnalyticsError.invalidArgument(message: "Invalid Arguments \(args ?? "null")")
    }
    Analytics.setDebugLogEnabled(enabled)
  }
  
  private func setEnabled(_ args: Any?) throws {
    guard let enabled = args as? Bool else {
      throw DIAnalyticsError.invalidArgument(message: "Invalid Arguments \(args ?? "null")")
    }
    Analytics.setEnabled(enabled)
  }
  
  private func getVersion() -> String {
    return Analytics.version
  }
}


enum DIAnalyticsError : Error {
  case unsupportedMethod(name: String)
  case invalidArgument(message: String)
}
