import 'dart:async';
import 'dart:io';

import 'package:di_analytics/un_implemented_di_analytics.dart';
import 'package:di_analytics/mobile_di_analytics.dart';

abstract class DIAnalytics {
  static late final DIAnalytics instance = (() =>
      Platform.isIOS || Platform.isAndroid
          ? MobileDIAnalytics()
          : UnimplementedDIAnalytics())();

  /// Configure analytics with default configuration. Must be called 1 before logEvent.
  ///
  /// [apiKey] and [trackingUrl] are required. Others are optional.
  /// Events will be submit by batch every [submitInterval] seconds. Default: 1min.
  /// [submitTimeoutInterval] is timeout value for submit api. Default 2mins.
  /// [maxSizeOfLogFile] is max size in bytes per log file. Default 1MB.
  /// [maxNumberOfLogFile] is max number of log file stored. Default 10.
  /// If there are no interaction after [sessionExpireTime], it will be expired. Default 10mins.
  /// if [submitLogOnWifiOnly] is true, sdk only submit log when connected to a wifi network.
  /// if [queueEnabled] set false, log will be submit immediately. Default true.
  Future configure({
    required String apiKey,
    required String trackingUrl,
    int? submitInterval,
    int? submitTimeoutInterval,
    int? maxSizeOfLogFile,
    int? maxNumberOfLogFile,
    int? sessionExpireTime,
    bool? submitLogOnWifiOnly,
    bool? queueEnabled,
  });

  /// Log an event.
  ///
  /// Event [name] is required. An optional [parameters] Map with primitive types values
  /// can be provided.
  Future logEvent(String name, {Map<String, dynamic>? parameters});

  /// Track current logged-in user.
  ///
  /// [userId] is app user identifier. Set [userId] = null to clear.
  /// An optional [info] Map with primitive types values can be provided.
  Future setUser(String? userId, {Map<String, dynamic>? info});

  /// Clears all analytics data for this instance from the device.
  Future resetAnalyticsData();

  /// Dispatch events immediately.
  Future dispatchEvents();

  /// Add Parameters to every logged events.
  ///
  /// [parameters] only accept primitive types values
  /// To remove a previous set parameter, set its value = nil
  Future addSharedParameters(Map<String, dynamic> parameters);

  /// Debug console log [enabled], default: false.
  Future setDebugLogEnabled(bool enabled);

  /// Enable or disable tracking. Default true.
  Future setEnabled(bool enabled);

  /// Get current sdk version.
  ///
  /// Return sdk version as String.
  Future<String> getVersion();
}
