import 'dart:io';

import 'package:di_analytics/di_analytics.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';

class MobileDIAnalytics implements DIAnalytics {
  static const MethodChannel _channel =
      MethodChannel('co.datainsider.analytics');

  /// Configure analytics with default configuration. Must be called 1 before logEvent.
  ///
  /// [apiKey] and [trackingUrl] are required. Others are optional.
  /// Events will be submit by batch every [submitInterval] seconds. Default: 1min.
  /// [submitTimeoutInterval] is timeout value for submit api. Default 2mins.
  /// [maxSizeOfLogFile] is max size in bytes per log file. Default 1MB.
  /// [maxNumberOfLogFile] is max number of log file stored. Default 10.
  /// If there are no interaction after [sessionExpireTime], it will be expired. Default 10mins.
  /// if [submitLogOnWifiOnly] is true, sdk only submit log when connected to a wifi network.
  /// if [queueEnabled] set false, log will be submit immediately. Default true.
  @override
  Future configure({
    required String apiKey,
    required String trackingUrl,
    int? submitInterval,
    int? submitTimeoutInterval,
    int? maxSizeOfLogFile,
    int? maxNumberOfLogFile,
    int? sessionExpireTime,
    bool? submitLogOnWifiOnly,
    bool? queueEnabled,
  }) async {
    try {
      return await _channel.invokeMethod("configure", {
        "apiKey": apiKey,
        "trackingUrl": trackingUrl,
        if (submitInterval != null) "submitInterval": submitInterval,
        if (submitTimeoutInterval != null)
          "submitTimeoutInterval": submitTimeoutInterval,
        if (maxSizeOfLogFile != null) "maxSizeOfLogFile": maxSizeOfLogFile,
        if (maxNumberOfLogFile != null)
          "maxNumberOfLogFile": maxNumberOfLogFile,
        if (sessionExpireTime != null) "sessionExpireTime": sessionExpireTime,
        if (submitLogOnWifiOnly != null)
          "submitLogOnWifiOnly": submitLogOnWifiOnly,
        if (queueEnabled != null) "queueEnabled": queueEnabled,
      });
    } on PlatformException catch (e) {
      debugPrint("DIAnalytics configure error $e");
    }
  }

  /// Log an event.
  ///
  /// Event [name] is required. An optional [parameters] Map with primitive types values
  /// can be provided.
  @override
  Future logEvent(String name, {Map<String, dynamic>? parameters}) async {
    try {
      return await _channel.invokeMethod("logEvent", {
        "name": name,
        if (parameters != null) "parameters": parameters,
      });
    } on PlatformException catch (e) {
      debugPrint("DIAnalytics logEvent error $e");
    }
  }

  /// Track current logged-in user.
  ///
  /// [userId] is app user identifier. Set [userId] = null to clear.
  /// An optional [info] Map with primitive types values can be provided.
  @override
  Future setUser(String? userId, {Map<String, dynamic>? info}) async {
    try {
      return await _channel.invokeMethod("setUser", {
        "userId": userId,
        if (info != null) "info": info,
      });
    } on PlatformException catch (e) {
      debugPrint("DIAnalytics setUser error $e");
    }
  }

  /// Clears all analytics data for this instance from the device.
  @override
  Future resetAnalyticsData() async {
    try {
      return await _channel.invokeMethod("resetAnalyticsData");
    } on PlatformException catch (e) {
      debugPrint("DIAnalytics resetAnalyticsData error $e");
    }
  }

  /// Dispatch events immediately.
  @override
  Future dispatchEvents() async {
    try {
      return await _channel.invokeMethod("dispatchEvents");
    } on PlatformException catch (e) {
      debugPrint("DIAnalytics dispatchEvents error $e");
    }
  }

  /// Add Parameters to every logged events.
  ///
  /// [parameters] only accept primitive types values
  /// To remove a previous set parameter, set its value = nil
  @override
  Future addSharedParameters(Map<String, dynamic> parameters) async {
    try {
      return await _channel.invokeMethod("addSharedParameters", parameters);
    } on PlatformException catch (e) {
      debugPrint("DIAnalytics addSharedParameters error $e");
    }
  }

  /// Debug console log [enabled], default: false.
  @override
  Future setDebugLogEnabled(bool enabled) async {
    try {
      return await _channel.invokeMethod("setDebugLogEnabled", enabled);
    } on PlatformException catch (e) {
      debugPrint("DIAnalytics setDebugLogEnabled error $e");
    }
  }

  /// Enable or disable tracking. Default true.
  @override
  Future setEnabled(bool enabled) async {
    try {
      return await _channel.invokeMethod("setEnabled", enabled);
    } on PlatformException catch (e) {
      debugPrint("DIAnalytics setEnabled error $e");
    }
  }

  /// Get current sdk version.
  ///
  /// Return sdk version as String.
  @override
  Future<String> getVersion() async {
    try {
      return await _channel.invokeMethod("getVersion");
    } on PlatformException catch (e) {
      debugPrint("DIAnalytics getVersion error $e");
      return "unknown";
    }
  }
}
